
#include <stdio.h>
#include <stdlib.h>
#include <tdb.h>
#include <glib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

static TDB_CONTEXT * prefs_tdb = NULL;

void
prefs_init ()
{
  char * prefs_path;
  struct stat sbuf;
  
  prefs_path = g_get_home_dir ();
  prefs_path = g_strconcat (prefs_path, "/.sweep", NULL);

  if (stat (prefs_path, &sbuf) == -1) {
    switch (errno) {
    case ENOENT:
      if (mkdir (prefs_path, S_IRWXU) == -1) {
	if (errno != EEXIST) {
	  perror ("Error creating ~/.sweep");
	  exit (1);
	}
      }
      break;
    case EACCES:
      if (chmod (prefs_path, S_IRWXU) == -1) {
	perror ("Error setting permissions on ~/.sweep");
	exit (1);
      }
      break;
    default:
      perror ("Error on ~/.sweep");
      exit (1);
    }
  }

  prefs_path = g_strconcat (prefs_path, "/preferences.tdb", NULL);

  if (stat (prefs_path, &sbuf) == -1) {
    switch (errno) {
    case EACCES:
      if (chmod (prefs_path, S_IRUSR | S_IWUSR) == -1) {
	perror ("Error setting permissions on ~/.sweep/preferences.tdb");
	exit (1);
      }
      break;
    default:
      break;
    }
  } else {
    if (sbuf.st_mode != (S_IRUSR | S_IWUSR)) {
      if (chmod (prefs_path, S_IRUSR | S_IWUSR) == -1) {
	perror ("Error setting permissions on ~/.sweep/preferences.tdb");
	exit (1);
      }
    }
  }

  prefs_tdb = tdb_open (prefs_path, 0, 0, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);

  if (prefs_tdb == NULL) {
    perror ("Error opening ~/.sweep/preferences.tdb");
    exit (1);
    return;
  }
}

int
prefs_close (void)
{
  if (prefs_tdb == NULL) return 0;

  return tdb_close (prefs_tdb);
}

int *
prefs_get_int (char * key)
{
  TDB_DATA key_data, val_data;
  int nval, val;

  if (prefs_tdb == NULL) return NULL;

  key_data.dptr = key;
  key_data.dsize = strlen (key);

  val_data = tdb_fetch (prefs_tdb, key_data);

  if (val_data.dptr == NULL) {
    return NULL;
  }

  nval = * (int *)val_data.dptr;
  val = g_ntohl (nval);

  * (int *)val_data.dptr = val;

  return (int *)val_data.dptr;
}

int
prefs_set_int (char * key, int val)
{
  TDB_DATA key_data, val_data;
  int nval;

  if (prefs_tdb == NULL) return -1;

  key_data.dptr = key;
  key_data.dsize = strlen (key);

  nval = g_htonl (val);

  val_data.dptr = (char *)&nval;
  val_data.dsize = sizeof (int);

  return tdb_store (prefs_tdb, key_data, val_data, TDB_REPLACE);
}

char *
prefs_get_string (char * key)
{
  TDB_DATA key_data, val_data;

  if (prefs_tdb == NULL) return NULL;

  key_data.dptr = key;
  key_data.dsize = strlen (key);

  val_data = tdb_fetch (prefs_tdb, key_data);

  return (char *)val_data.dptr;
}

int
prefs_set_string (char * key, char * val)
{
  TDB_DATA key_data, val_data;

  if (prefs_tdb == NULL) return -1;

  key_data.dptr = key;
  key_data.dsize = strlen (key);

  val_data.dptr = val;
  val_data.dsize = strlen (val) + 1;

  return tdb_store (prefs_tdb, key_data, val_data, TDB_REPLACE);
}
